# -*- encoding: utf-8 -*-
"""
@File    :   file_processing.py
@Time    :   2022/02/17 11:51:44
@Author  :   Yimin Jing
@Contact :   jingym3@lenovo.com
@Desc    :   文件的读取和写入
"""

import os
import json
import glob
import yaml
import pandas as pd
from openpyxl import Workbook, load_workbook

    
def load_single_json(in_path, prefix=None):
    """
    @Desc: 加载Json文件，整个Json文件是一个python字典
    @Args: 
        in_path: Json文件路径
        prefix: 该字典的某个字段 (key，键)
    @Returns: 
        python字典或列表
    """
    with open(in_path, "r", encoding="utf-8") as f:
        if prefix:
            data = json.load(f)[prefix]
        else:
            data = json.load(f)
    return data
    

def load_multi_json(in_path):
    """
    @Desc: 加载Json文件，一行是一个python字典
    @Args: 
        in_path: Json文件路径
    @Returns:
        python列表
    """
    with open(in_path, "r", encoding="utf-8") as f:
        data = [json.loads(line.strip()) for line in f]
    return data
    

def write_json(data, out_path):
    """
    @Desc: 将python字典或字典列表写入Json文件
    @Args: 
        data: python字典或者字典列表
    @Returns: 
        输出json文件
    """
    with open(out_path, "w", encoding="utf-8") as f:
        if isinstance(data, list):
            for ele in data:
                if isinstance(ele, dict):
                    f.write("{}\n".format(json.dumps(ele, ensure_ascii=False)))
        elif isinstance(data, dict):
            f.write(json.dumps(data, ensure_ascii=False, indent=4))
        else:
            raise Exception("写入json数据的文件格式不正确")
    
    
def output_filenames(in_directory, file_type):
    """
    @Desc: 输出给定目录里的所有指定文件类型的绝对路径
    @Args: 
        in_directory: 给定目录
        file_type: 指定文件类型
    @Returns: 
        路径List
    """
    specific_directory = os.path.join(in_directory, "*.{}".format(file_type))
    return glob.glob(specific_directory)
    

def load_single_txt(in_path, keep_enter=False):
    """
    @Desc: 加载单个纯文本文件
    @Args: 
        in_path: 纯文本文件路径
        keep_enter: 用来确定是否保留每一行的换行符"\n"
    @Returns: 
        python列表
    """
    data = []
    with open(in_path, "r", encoding="utf-8") as f:
        for line in f:
            if keep_enter:
                data.append(line)
            else:
                if line.strip():
                    data.append(line.strip())
    return data


def load_multi_txt(in_directory, file_type, keep_enter=False):
    """
    @Desc: 加载给定目录里的所有纯文本文件
    @Args: 
        in_directory: 给定目录
        file_type: 文件类型
        keep_enter: 是否保留换行符
    @Returns: 
        python列表
    """
    filenames = output_filenames(in_directory, file_type)
    data = []
    for filename in filenames:
        res = load_single_txt(filename, keep_enter=keep_enter)
        data.extend(res)
    return data


def write_txt(data, out_path):
    """
    @Desc: 将python列表数据写入纯文本文件
    @Args: 
        data: ["今天天气晴朗", "适合爬山"]
        out_path: 输出的额纯文本文件路径
    @Returns: 
    """
    with open(out_path, "w", encoding="utf-8") as f:
        for ele in data:
            f.write(ele.strip() + "\n")


def load_yaml(in_path):
    """
    @Desc: 加载yaml参数配置文件
    @Args: 
        in_path: yaml文件路径 
    @Returns: 
        python字典
    """
    with open(in_path, "r", encoding="utf-8") as f:
        data = yaml.load(f, Loader=yaml.FullLoader)
    return data


def write_excel(data, out_path, sheet_name="Sheet"):

    """
    @Desc: 将数据写入excel文件
    @Args: 
        data: {"列名1":[ele1, ele2], "列名2": [ele1, ele2]}
        out_path: *.xlsx
        sheet_name: 表单名
    @Returns: 
        excel文件
    """
    if os.path.exists(out_path):
        book = load_workbook(out_path)
        writer = pd.ExcelWriter(out_path, engine="openpyxl", mode="a")
        writer.book = book
    else:
        book = Workbook()
        writer = pd.ExcelWriter(out_path, engine="openpyxl", mode="w")
    df = pd.DataFrame(data)
    df.to_excel(writer, sheet_name=sheet_name, index=False)
    writer.save()
    



# -*- encoding: utf-8 -*-
'''
@File   :   data_processing.py
@Time   :   2021/07/05 18:14:52
@Author :   Yimin Jing
@Email  :   yymmjing@gmail.com
@Desc   :   划分数据集
'''

import random
import math


def split_dataset(corpus: list, ratio: str, is_shuffle=True) -> list:
    """
    @Desc: 给定一个原始数据集，按照比例将其划分为训练集、验证集、测试集
    @Args:
        corpus: 给定数据集，python列表
        ratio: 数据集划分比例，7:2:1或者6:4等
        is_shuffle: 是否打乱数据集，默认True
    @Returns:
        若是7:2:1等三个比例，则返回训练集、验证集和测试集
        若是6:4等两个比例，则返回训练集和测试集
    """
    if is_shuffle:
        random.shuffle(corpus)
    ratio = [int(ele) for ele in ratio.split(":")]
    assert sum(ratio) == 10, "ratio必须按10来划分，7:2:1或者6:4"
    length = len(corpus)
    
    # ratio等于3，即将corpus划分为训练集、验证集和测试集
    if len(ratio) == 3:
        index0 = math.ceil(length * ratio[0] / 10)
        index1 = math.ceil(length * ratio[1] / 10)
        index2 = length - index0 - index1
        return corpus[:index0], corpus[index0:index0+index1], corpus[-index2:]
    # ratio等于2，即将corpus划分为训练集、测试集
    elif len(ratio) == 2:
        index0 = math.ceil(length * ratio[0] / 10)
        index1 = length - index0
        return corpus[:index0], corpus[-index1:]
    else:
        raise Exception("ratio设置不合理，ratio=2或3")


def split_dataset_by_class(corpus: list, ratio: str, cate="category", is_shuffle=True) -> list:
    """
    @Desc: corpus中每个元素是dict，按照类别进行数据切分
    @Args:
        corpus: 给定数据集，python列表，列表中每一个元素是dict
        ratio: 数据集划分比例，7:2:1或者6:4等
        cate: 按dict的哪个字段来划分数据集
        is_shuffle: 是否打乱数据集，默认True
    @Returns:
        若是7:2:1等三个比例，则返回训练集、验证集和测试集
        若是6:4等两个比例，则返回训练集和测试集
    """
    _ratio = [int(ele) for ele in ratio.split(":")]
    assert sum(_ratio) == 10, "ratio必须按10来划分，7:2:1或者6:4"

    cate_corpus = {}
    for sample in corpus:
        category = sample[cate]
        if category not in cate_corpus:
            cate_corpus[category] = []
            cate_corpus[category].append(sample)
        else:
            cate_corpus[category].append(sample)
    
    train_data = []
    dev_data = []
    test_data = []
    if len(_ratio) == 3:
        for key, value in cate_corpus.items():
            assert len(value) >= 10, "{}类别样例数仅为：{}".format(key, len(value)) 
            print("{}类别的数据样例为：{}".format(key, len(value)))
            train, dev, test = split_dataset(value, ratio, is_shuffle=False)
            train_data.extend(train)
            dev_data.extend(dev)
            test_data.extend(test)
        if is_shuffle:
            random.shuffle(train_data)
            random.shuffle(dev_data)
            random.shuffle(test_data)
        return train_data, dev_data, test_data
    elif len(_ratio) == 2:
        for key, value in cate_corpus.items():
            print("{}类别的数据样例为：{}".format(key, len(value)))
            train, test = split_dataset(value, ratio, is_shuffle=False)
            train_data.extend(train)
            test_data.extend(test)
        if is_shuffle:
            random.shuffle(train_data)
            random.shuffle(test_data)
        return train_data, test_data
    else:
        raise Exception("ratio设置不合理，ratio=2或3")


    





# -*- encoding: utf-8 -*-
'''
@File    :   setup.py
@Time    :   2021/11/17 18:13:17
@Author  :   Sharejing
@Contact :   yymmjing@gmail.com
@Desc    :   
'''

from setuptools import setup


setup(
    name="takin",
    version="1.1.2",
    author="Yimin Jing",
    author_email="yymmjing@gmail.com",
    description="A Python Toolkit for File Processing, Text Cleaning and Data Splitting",
    long_description=open("README.md").read(),
    license="MIT",
    url="https://github.com/sharejing/Takin",
    packages=["takin"],
    keywords="NLP text cleaning file processing data splitting",
    python_requires=">=3.6.0",
    install_requires=[
        "pyyaml==6.0",
        "pandas==1.3.4",
        "openpyxl==3.0.9",
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Topic :: Scientific/Engineering",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
)


# -*- encoding: utf-8 -*-
'''
@File   :   data_cleaning.py
@Time   :   2021/04/12 18:28:06
@Author :   ShareJing
@Email  :   yymmjing@gmail.com
@Desc   :   文本清洗函数集
'''

import re
from string import punctuation
from itertools import groupby


def delete_escape_character(text, lang="zh", add_punc=False):
    """
    @Desc: 删除文本中的转移字符
    @Args: 
        text: 指定文本
        lang: 该文本对应的语言
        add_punc: 是否补全标点符号
    @Returns: 
        删除后的文本
    """
    list_text = list(text)
    for idx, char in enumerate(list_text):
        if char in ["\t", "\v", "\f", "\r", "\n"]:
            list_text[idx] = ""
            if add_punc:
                if lang == "zh":
                    if list_text[idx-1] != "。":
                        list_text[idx] = "。"
                else:
                    if list_text[idx-1] != ".":
                        list_text[idx] = ". "
    return "".join(list_text)


def delete_extra_whitespace(text, lang="zh"):
    """
    @Desc: 删除文本中的多余空白
    @Args:
        text: 指定文本
        lang: 该文本对应的语言
    @Returns:
        删除后的文本
    """
    if lang == "zh":
        return "".join(text.split())
    else:
        list_text = list(" ".join(text.split()))
        for index, ele in enumerate(list_text):
            if index != 0 and ele in punctuation:
                if list_text[index-1] == " ":
                    list_text[index-1] = ""
        return "".join(list_text)


def delete_digit(text):
    """
    @Desc: 删除文本中的数字 (百分数、分数、小数、整数)
    @Args:
        text: 指定文本
    @Returns:
        删除后的文本
    """
    # 百分数、分数、小数、整数
    DIGIT_PATTERN = r"\d+\.?\d+?%(?#百分数)|\d/\d+(?#分数)|-?\d+\.?\d+(?#小数和整数)"
    return re.sub(DIGIT_PATTERN, "", text)


def delete_punctuation(text):
    """
    @Desc: 删除文本中的所有标点符号 (运算符保留)
    @Args: None
    @Returns: None
    """
    CHINESE_PUNC = r"(，|。|、|？|！|“|”|‘|’|；|：|《|》|￥|（|）|〈|〉|·)"
    ENGLISH_PUNC = r"(,|!|\?|'|\"|…|-|\$|@|\[|\]|{|}|_|\^|#|<|>|\(|\)|&|\\|;|:)"
    SPECIAL_PUNC = r"(╔|ˊ|η|●|®|•|~|★|\||▶|–)"
    OPERATION_PATTERN = r"(?<!\d)(\.|\+|\=|\*|/|%)(?!\d)"

    text = re.sub(CHINESE_PUNC, "", text)
    text = re.sub(ENGLISH_PUNC, "", text)
    text = re.sub(SPECIAL_PUNC, "", text)
    text = re.sub(OPERATION_PATTERN, "", text)
    return text


def delete_letter(text):
    """
    @Desc: 删除所有英文字母
    @Args:
        text: 指定文本
    @Returns:
        删除后的文本
    """
    LETTER_PATTERN = r"[A-Za-z]+"
    return re.sub(LETTER_PATTERN, "", text)


def delete_chinese(text):
    """
    @Desc: 删除所有中文字符
    @Args:
        text: 指定文本
    @Returns:
        删除后的文本
    """
    CHINESE_PATTERN = r"[\u4e00-\u9fa5]+"
    return re.sub(CHINESE_PATTERN, "", text)


def delete_bracket(text):
    """
    @Desc: 删除括号以及括号里的内容，包括 <*>、(*)、（*）、【*】、[*]、{*}
    @Args:
        text: 指定文本
    @Returns:
        删除后的文本
    """
    BRACKET_PATTERN = r"<.*>|\(.*\)|（.*）|【.*】|\[.*\]|{.*}"
    return re.sub(BRACKET_PATTERN, "", text)


def delete_series_number(text):
    """
    @Desc: 删除文本中的序号
    @Args:
        text: 指定文本
    @Returns:
        删除后的文本
    """ 
    # 1.|(2)|(2).|4)|4)、|（一）|（二）、 
    SERIES_PATTERN = r"\d+\.|\(\d+\)\.?|\d+\)(\.|、)?|（(一|二|三|四|五|六|七|八|九|十|百|千|万|亿)+）(\.|、)?"
    return re.sub(SERIES_PATTERN, "", text)


def delete_repeated_punc(text):
    """
    @Desc: 连续重复的标点符号只保留一次
    @Args:
        text: 指定文本
    @Returns:
        删除后的文本
    """
    punctuations = [';', ':', ',', '.', '"', '?', '!', "'", '·', '！', '？', '；', '，', '。', '：', '“', '”', '、', '‘', '’', '《', '》', '[', '╔', 'ˊ', '〉', '〈', '–', 'η', '●', '®', '·', '•', '-', '~', '#', '/', '*', '&', '$', '|', '★', '▶', '>', '<', '\\', '^', '@', '+', '[', '=', ']', '(', ')', '（', '）', '{', '%', '_', '}', '?', '…']
    t = [[k, len(list(g))] for k, g in groupby(text)]
    res = ""
    for ele in t:
        char = ele[0]
        count = ele[1]
        if char in punctuations and count > 1:
            count = 1
        res += char * count
    return res